@@ -6,10 +6,10 @@ import os
 import uuid
 import warnings
 from abc import abstractmethod
-from collections import namedtuple
 from contextlib import contextmanager
 
 import mlflow
+import mlflow.utils.autologging_utils
 from mlflow.entities.run_status import RunStatus
 from mlflow.tracking.client import MlflowClient
 from mlflow.utils import gorilla
@@ -23,6 +23,8 @@ from mlflow.utils.mlflow_tags import MLFLOW_AUTOLOGGING
 
 _AUTOLOGGING_TEST_MODE_ENV_VAR = "MLFLOW_AUTOLOGGING_TESTING"
 
+_AUTOLOGGING_PATCHES = {}
+
 
 def try_mlflow_log(fn, *args, **kwargs):
     """
@@ -180,7 +182,8 @@ def with_managed_run(autologging_integration, patch_function, tags=None):
           with the `FAILED` if an unhandled exception is thrown during function execution
 
     Note that, if nested runs or non-fluent runs are created by `patch_function`, `patch_function`
-    is responsible for terminating them by the time it terminates (or in the event of an exception).
+    is responsible for terminating them by the time it terminates
+    (or in the event of an exception).
 
     :param autologging_integration: The autologging integration associated
                                     with the `patch_function`.
@@ -191,9 +194,7 @@ def with_managed_run(autologging_integration, patch_function, tags=None):
     """
 
     def create_managed_run():
-        managed_run = mlflow.start_run()
-        if tags:
-            try_mlflow_log(mlflow.set_tags, tags)
+        managed_run = mlflow.start_run(tags=tags)
         _logger.info(
             "Created MLflow autologging run with ID '%s', which will track hyperparameters,"
             " performance metrics, model artifacts, and lineage information for the"
@@ -335,10 +336,10 @@ def safe_patch(
         # during model serialization by ML frameworks such as scikit-learn
         is_silent_mode = get_autologging_config(autologging_integration, "silent", False)
         with set_mlflow_events_and_warnings_behavior_globally(
-            # MLflow warnings emitted during autologging training sessions are likely not actionable
-            # and result from the autologging implementation invoking another MLflow API.
-            # Accordingly, we reroute these warnings to the MLflow event logger with level WARNING
-            # For reference, see recommended warning and event logging behaviors from
+            # MLflow warnings emitted during autologging training sessions are likely not
+            # actionable and result from the autologging implementation invoking another MLflow
+            # API. Accordingly, we reroute these warnings to the MLflow event logger with level
+            # WARNING For reference, see recommended warning and event logging behaviors from
             # https://docs.python.org/3/howto/logging.html#when-to-use-logging
             reroute_warnings=True,
             disable_event_logs=is_silent_mode,
@@ -347,9 +348,9 @@ def safe_patch(
             # non-MLflow Warnings emitted during the autologging preamble (before the original /
             # underlying ML function is called) and postamble (after the original / underlying ML
             # function is called) are likely not actionable and result from the autologging
-            # implementation invoking an API from a dependent library. Accordingly, we reroute these
-            # warnings to the MLflow event logger with level WARNING. For reference, see recommended
-            # warning and event logging behaviors from
+            # implementation invoking an API from a dependent library. Accordingly, we reroute
+            # these warnings to the MLflow event logger with level WARNING. For reference, see
+            # recommended warning and event logging behaviors from
             # https://docs.python.org/3/howto/logging.html#when-to-use-logging
             reroute_warnings=True,
             disable_warnings=is_silent_mode,
@@ -366,9 +367,16 @@ def safe_patch(
             user_created_fluent_run_is_active = (
                 mlflow.active_run() and not _AutologgingSessionManager.active_session()
             )
+            active_session_failed = (
+                _AutologgingSessionManager.active_session() is not None
+                and _AutologgingSessionManager.active_session().state == "failed"
+            )
 
-            if autologging_is_disabled(autologging_integration) or (
-                user_created_fluent_run_is_active and exclusive
+            if (
+                active_session_failed
+                or autologging_is_disabled(autologging_integration)
+                or (user_created_fluent_run_is_active and exclusive)
+                or mlflow.utils.autologging_utils._AUTOLOGGING_GLOBALLY_DISABLED
             ):
                 # If the autologging integration associated with this patch is disabled,
                 # or if the current autologging integration is in exclusive mode and a user-created
@@ -430,8 +438,8 @@ def safe_patch(
                             original_has_been_called = True
 
                             nonlocal original_result
-                            # Show all non-MLflow warnings as normal (i.e. not as event logs) during
-                            # original function execution, even if silent mode is enabled
+                            # Show all non-MLflow warnings as normal (i.e. not as event logs)
+                            # during original function execution, even if silent mode is enabled
                             # (`silent=True`), since these warnings originate from the ML framework
                             # or one of its dependencies and are likely relevant to the caller
                             with set_non_mlflow_warnings_behavior_for_current_thread(
@@ -483,6 +491,8 @@ def safe_patch(
                     else:
                         patch_function(call_original, *args, **kwargs)
 
+                    session.state = "succeeded"
+
                     try_log_autologging_event(
                         AutologgingEventLogger.get_logger().log_patch_function_success,
                         session,
@@ -492,6 +502,8 @@ def safe_patch(
                         kwargs,
                     )
                 except Exception as e:
+                    session.state = "failed"
+
                     # Exceptions thrown during execution of the original function should be
                     # propagated to the caller. Additionally, exceptions encountered during test
                     # mode should be reraised to detect bugs in autologging implementations
@@ -530,13 +542,35 @@ def safe_patch(
                 else:
                     return original(*args, **kwargs)
 
-    _wrap_patch(destination, function_name, safe_patch_function)
+    new_patch = _wrap_patch(destination, function_name, safe_patch_function)
+    _store_patch(autologging_integration, new_patch)
+
+
+def revert_patches(autologging_integration):
+    """
+    Reverts all patches on the specified destination class for autologging disablement
+    purposes.
+
+    :param autologging_integration: The name of the autologging integration associated with the
+                                    patch. Note: If called via fluent api
+                                    (`autologging_integration="mlfow"`), then revert all patches
+                                    for all active autologging integrations.
+    """
+    for patch in _AUTOLOGGING_PATCHES.get(autologging_integration, []):
+        gorilla.revert(patch)
+
+    _AUTOLOGGING_PATCHES.pop(autologging_integration, None)
 
 
 # Represents an active autologging session using two fields:
 # - integration: the name of the autologging integration corresponding to the session
 # - id: a unique session identifier (e.g., a UUID)
-AutologgingSession = namedtuple("AutologgingSession", ["integration", "id"])
+# - state: the state of AutologgingSession, will be one of running/succeeded/failed
+class AutologgingSession:
+    def __init__(self, integration, id_):
+        self.integration = integration
+        self.id = id_
+        self.state = "running"
 
 
 class _AutologgingSessionManager:
@@ -604,6 +638,22 @@ def _wrap_patch(destination, name, patch, settings=None):
 
     patch = gorilla.Patch(destination, name, wrapped, settings=settings)
     gorilla.apply(patch)
+    return patch
+
+
+def _store_patch(autologging_integration, patch):
+    """
+    Stores a patch for a specified autologging_integration class. Later to be used for being able
+    to revert the patch when disabling autologging.
+
+    :param autologging_integration: The name of the autologging integration associated with the
+                                    patch.
+    :param patch: The patch to be stored.
+    """
+    if autologging_integration in _AUTOLOGGING_PATCHES:
+        _AUTOLOGGING_PATCHES[autologging_integration].add(patch)
+    else:
+        _AUTOLOGGING_PATCHES[autologging_integration] = set([patch])
 
 
 def _validate_autologging_run(autologging_integration, run_id):
